@@ -674,3 +674,33 @@ namespace InitializerList {
     clang_analyzer_eval(list->usedInitializerList); // expected-warning{{UNKNOWN}}
   }
 }
+
+namespace PR19579 {
+  class C {};
+
+  void f() {
+    C();
+    int a;
+
+    extern void use(int);
+    use(a); // expected-warning{{uninitialized}}
+  }
+
+  void g() {
+    struct S {
+      C c;
+      int i;
+    };
+    
+    // This order triggers the initialization of the inner "a" after the
+    // constructor for "C" is run, which used to confuse the analyzer
+    // (is "C()" the initialization of "a"?).
+    struct S s = {
+      C(),
+      ({
+        int a, b = 0;
+        0;
+      })
+    };
+  }
+}
